
 //This example demonstrates how to create nodes and elements using Zinc APIs and how to group/select/highlight
 //them programmatically.
 	
function show()
{
	document.getElementById('zincplugin').style.display='';
}

function hide()
{
	document.getElementById('zincplugin').style.display='none';
}

function selectNode(regionName, identifier)
{
	var zincPlugin = document.getElementById('zincplugin');
	var rootRegion = zincPlugin.context.getDefaultRegion();
	var childRegion = rootRegion.findChildByName(regionName);
	var graphicsModule = zincPlugin.context.getDefaultGraphicsModule();
	var rootRendition = graphicsModule.getRendition(rootRegion);
	/* get the selection group field of from root rendition */
	var selectionGroup = rootRendition.getSelectionGroup();
	/* if no existing selection group, create a new one then set the seleciton group to
		root rendition*/
	if (selectionGroup == null)
	{
		var fieldModule = rootRegion.getFieldModule(); 
		selectionGroup = fieldModule.createGroup();
		rootRendition.setSelectionGroup(selectionGroup);
	}
	/* try to get the selection group for child region*/
	var subRegionGroup = selectionGroup.getSubregionGroup(childRegion);
	/* create a new one if no group is found on child region */
	if (subRegionGroup == null)
		subRegionGroup = selectionGroup.createSubregionGroup(childRegion);
	/* get the master nodeset from child region */ 
	var nodeset = childRegion.getFieldModule().findNodesetByName("cmiss_nodes");
	/* get the node group from the group field */
	var nodeGroup = subRegionGroup.getNodeGroup(nodeset);
	/* create a new one if node group is not created already */
	if (nodeGroup == null)
		nodeGroup = subRegionGroup.createNodeGroup(nodeset);
	/* find the node we want to select with the identifier then add it into the node group through nodeset */
	var node = nodeset.findNodeByIdentifier(identifier);
	var nodesetGroup = nodeGroup.getNodeset();
	nodesetGroup.addNode(node);
	/* the selected node should now be highlighted */
}

/* this is similar to the selectNode function, only it removes the node from the nodesetGroup */
function UnselectNode2(regionName)
{
	var zincPlugin = document.getElementById('zincplugin');
	var rootRegion = zincPlugin.context.getDefaultRegion();
	var childRegion = rootRegion.findChildByName(regionName);
	var graphicsModule = zincPlugin.context.getDefaultGraphicsModule();
	var rootRendition = graphicsModule.getRendition(rootRegion);
	var fieldModule = rootRegion.getFieldModule(); 
	var selectionGroup = rootRendition.getSelectionGroup();
	if (selectionGroup != null)
	{
		var subRegionGroup = selectionGroup.getSubregionGroup(childRegion);
		if (subRegionGroup == null)
			subRegionGroup = selectionGroup.createSubregionGroup(childRegion);
		var nodeset = childRegion.getFieldModule().findNodesetByName("cmiss_nodes");
		var nodeGroup = subRegionGroup.getNodeGroup(nodeset);
		if (nodeGroup == null)
			nodeGroup = subRegionGroup.createNodeGroup(nodeset);
		var node = nodeset.findNodeByIdentifier(2);
		var nodesetGroup = nodeGroup.getNodeset();
		nodesetGroup.removeNode(node);
	}
}

/* please see selectNode for more description */
function selectElement(regionName, identifier)
{
	var zincPlugin = document.getElementById('zincplugin');
	var rootRegion = zincPlugin.context.getDefaultRegion();
	var childRegion = rootRegion.findChildByName(regionName);
	var graphicsModule = zincPlugin.context.getDefaultGraphicsModule();
	var rootRendition = graphicsModule.getRendition(rootRegion);
	var fieldModule = rootRegion.getFieldModule(); 
	var selectionGroup = rootRendition.getSelectionGroup();
	if (selectionGroup == null)
	{
		selectionGroup = fieldModule.createGroup();
		rootRendition.setSelectionGroup(selectionGroup);
	}
	var subRegionGroup = selectionGroup.getSubregionGroup(childRegion);
	if (subRegionGroup == null)
		subRegionGroup = selectionGroup.createSubregionGroup(childRegion);
	var mesh = childRegion.getFieldModule().findMeshByDimension(2);
	var elementGroup = subRegionGroup.getElementGroup(mesh);
	if (elementGroup == null)
		elementGroup = subRegionGroup.createElementGroup(mesh);
	var element = mesh.findElementByIdentifier(identifier);
	var meshGroup = elementGroup.getMesh();
	meshGroup.addElement(element);
}

function unselectElement(regionName, identifier)
{
	var zincPlugin = document.getElementById('zincplugin');
	var rootRegion = zincPlugin.context.getDefaultRegion();
	var childRegion = rootRegion.findChildByName(regionName);
	var graphicsModule = zincPlugin.context.getDefaultGraphicsModule();
	var rootRendition = graphicsModule.getRendition(rootRegion);
	var fieldModule = rootRegion.getFieldModule(); 
	var selectionGroup = rootRendition.getSelectionGroup();
	if (selectionGroup != null)
	{
		var subRegionGroup = selectionGroup.getSubregionGroup(childRegion);
		if (subRegionGroup == null)
			subRegionGroup = selectionGroup.createSubregionGroup(childRegion);
		var mesh = childRegion.getFieldModule().findMeshByDimension(2);
		var elementGroup = subRegionGroup.getElementGroup(mesh);
		if (elementGroup == null)
			elementGroup = subRegionGroup.createElementGroup(mesh);
		var element = mesh.findElementByIdentifier(identifier);
		var meshGroup = elementGroup.getMesh();
		meshGroup.removeElement(element);
	}
}

/* this function select the whole region */
function selectRegion(regionName)
{
	var zincPlugin = document.getElementById('zincplugin');
	var rootRegion = zincPlugin.context.getDefaultRegion();
	var childRegion = rootRegion.findChildByName(regionName);
	var graphicsModule = zincPlugin.context.getDefaultGraphicsModule();
	var rootRendition = graphicsModule.getRendition(rootRegion);
	var fieldModule = rootRegion.getFieldModule(); 
	var selectionGroup = rootRendition.getSelectionGroup();
	if (selectionGroup == null)
	{
		selectionGroup = fieldModule.createGroup();
		rootRendition.setSelectionGroup(selectionGroup);
	}
	/* add the entire region to the selection group field thus selecting the whole region */
	selectionGroup.addRegion(childRegion);
}

/* unselect the whole region */
function unselectRegion(regionName)
{
	var zincPlugin = document.getElementById('zincplugin');
	var rootRegion = zincPlugin.context.getDefaultRegion();
	var childRegion = rootRegion.findChildByName(regionName);
	var graphicsModule = zincPlugin.context.getDefaultGraphicsModule();
	var rootRendition = graphicsModule.getRendition(rootRegion);
	var selectionGroup = rootRendition.getSelectionGroup();
	if (selectionGroup != null)
	{
		/* check rather the region to be unselected is in the current selection group,
			if so remove the entire region selection */
		if (selectionGroup.containsRegion(childRegion))
		{
			selectionGroup.removeRegion(childRegion);
		}
		else
		{
			/* if the entire region is not in the group field then try to clean up the selected nodes
				or elements from that region */
			var subRegionGroup = selectionGroup.getSubregionGroup(childRegion);
			if (subRegionGroup != null)
				subRegionGroup.clear();
		}
		/* remove empty subgroup from the selection group field */
		selectionGroup.removeEmptySubgroups();
	}
}

/* remove all selections */
function clearAll()
{
	var zincPlugin = document.getElementById('zincplugin');
	var rootRegion = zincPlugin.context.getDefaultRegion();
	var graphicsModule = zincPlugin.context.getDefaultGraphicsModule();
	var rootRendition = graphicsModule.getRendition(rootRegion);
	var selectionGroup = rootRendition.getSelectionGroup();
	if (selectionGroup != null && (1 != selectionGroup.isEmpty()))
	{
		selectionGroup.clear();
		var childRegion = rootRegion.getFirstChild();
		while (childRegion != null)
		{
			childRegion = childRegion.getNextSibling();
		}
		selectionGroup.removeEmptySubgroups();
	}
}

/*****************************************************************************//**
* The following function creates a single element linear lagrange cube with 8 nodes
* using the finite element APIs provided in cmgui.
*/
function createCube()
{
	var zincPlugin = document.getElementById('zincplugin');
	var rootRegion = zincPlugin.context.getDefaultRegion();
	var region = rootRegion.createChild("cube");
	var fieldModule = region.getFieldModule();
	
	fieldModule.beginChange();
	
	var coordinatesField = fieldModule.createFiniteElement(3);
	coordinatesField.setName("coordinates");
	coordinatesField.setAttributeInteger(coordinatesField.ATTRIBUTE.IS_COORDINATE, 1);
	coordinatesField.setAttributeInteger(coordinatesField.ATTRIBUTE.IS_MANAGED, 1);
	
	var nodeset = fieldModule.findNodesetByName("cmiss_nodes");
	var nodeTemplate1 = nodeset.createNodeTemplate();
	nodeTemplate1.defineField(coordinatesField);
	var node_coordinates =
	[	[ 0, 0, 0 ],
		[ 1, 0, 0 ],
		[ 0, 1, 0 ],
		[ 1, 1, 0 ],
		[ 0, 0, 1 ],
		[ 1, 0, 1 ],
		[ 0, 1, 1 ],
		[ 1, 1, 1 ]
	];
	var fieldCache = fieldModule.createCache();
	var i = 0;
	var node;
	for (i=0; i<8; i++)
	{
		node = nodeset.createNode(i+1, nodeTemplate1);
		fieldCache.setNode(node);
		coordinatesField.assignReal(fieldCache, /*number_of_values*/3, node_coordinates[i]);
		node = null;
	}
	var tempCoordinatesField = fieldModule.createFiniteElement(3);
	tempCoordinatesField.setName("rc_coordinates");
	tempCoordinatesField.setAttributeInteger(coordinatesField.ATTRIBUTE.IS_COORDINATE, 1);
	tempCoordinatesField.setAttributeInteger(coordinatesField.ATTRIBUTE.IS_MANAGED, 1);
	var nodeTemplate1 = nodeset.createNodeTemplate();
	nodeTemplate1.defineField(tempCoordinatesField);
	for (i=0; i<8; i++)
	{
		node = nodeset.findNodeByIdentifier(i+1);
		node.merge(nodeTemplate1);
		fieldCache.setNode(node);
		tempCoordinatesField.assignReal(fieldCache, /*number_of_values*/3, node_coordinates[i]);
		node = null;
	}
	
	fieldCache = null;
	nodeTemplate1 = null;
	
	var mesh = fieldModule.findMeshByDimension(3);
	var elementTemplate = mesh.createElementTemplate();
	elementTemplate.setShapeType(elementTemplate.SHAPE_TYPE.CUBE);
	elementTemplate.setNumberOfNodes(8);
	var cubicBasis = fieldModule.createElementBasis(3, fieldModule.BASIS_FUNCTION_TYPE.LINEAR_LAGRANGE);
	var cube_local_node_indexes = [ 1, 2, 3, 4, 5, 6, 7, 8];
	elementTemplate.defineFieldSimpleNodal(coordinatesField,
		-1, cubicBasis, 8, cube_local_node_indexes);
	cubicBasis = null;
	
	/* create element */
	i = 0;
	for (i=1; i<=8; i++)
	{
		node = nodeset.findNodeByIdentifier(i);
		elementTemplate.setNode(i, node);
		node = null;
	}
	fieldModule.defineField("annotation", "constant  10");
	mesh.defineElement(0, elementTemplate);
	elementTemplate = null;
	mesh = null;
	nodeset = null;
	coordinatesField = null;
	fieldModule.endChange();
	field_module = null;
	region = null;
}

function drawGraphicForRendition(targetRendition)
{
	/* use execute command to draw the graphics for target rendition */
	targetRendition.beginChange();
	targetRendition.removeAllGraphics();
	var surfaces = targetRendition.createGraphic(targetRendition.GRAPHIC_TYPE.SURFACES);
	surfaces.define("coordinate coordinates select_on material default selected_material default_selected");
	var nodes = targetRendition.createGraphic(targetRendition.GRAPHIC_TYPE.NODE_POINTS);
	nodes.define("coordinate coordinates select_on material blue selected_material default_selected glyph sphere size '0.1*0.1*0.1'");
	//targetRendition.executeCommand("point glyph empty general label annotation centre -0.95,0.9,0.0 select_on material default selected_material default normalised_window_fit_left");
	targetRendition.endChange();
}

function createCubeGraphics()
{
	var zincPlugin = document.getElementById('zincplugin');
	var rootRegion = zincPlugin.context.getDefaultRegion();
	var child = rootRegion.findChildByName("cube");
	var childFieldModule = child.getFieldModule(); 
	childFieldModule.defineAllFaces();
	var graphicsModule = zincPlugin.context.getDefaultGraphicsModule();
	var rendition = graphicsModule.getRendition(child);
	drawGraphicForRendition(rendition);
	zincPlugin.sceneViewer.viewAll();
}

/* create a single tetrahedral element */
function createTetrahedron()
{
	var zincPlugin = document.getElementById('zincplugin');
	/* get the default region */
	var rootRegion = zincPlugin.context.getDefaultRegion();
	/* create a new region and we name it tetrahedron then get the
		field module from it */
	var region = rootRegion.createChild("tetrahedron");
	var fieldModule = region.getFieldModule();
	fieldModule.beginChange();
	
	/* create a 3 components finite element field, name it coordinates and make sure it is
		a coordinate field */
	var coordinatesField = fieldModule.createFiniteElement(3);
	coordinatesField.setName("coordinates");
	coordinatesField.setAttributeInteger(coordinatesField.ATTRIBUTE.IS_COORDINATE, 1);
	coordinatesField.setAttributeInteger(coordinatesField.ATTRIBUTE.IS_MANAGED, 1);
	
	/* find a nodeset using the field module APIs, create a template for node creation and
		define the coordinates field in this template */
	var nodeset = fieldModule.findNodesetByName("cmiss_nodes");
	var nodeTemplate1 = nodeset.createNodeTemplate();
	nodeTemplate1.defineField(coordinatesField);
	var node_coordinates =
	[	[ 2, 0, 0 ],
		[ 3, 0, 0 ],
		[ 2, 1, 0 ],
		[ 2.4, 0.4, 1 ]
	];
	/* create a field cache for defining/evaluating a field  */
	var fieldCache = fieldModule.createCache();
	var i = 0;
	var node;
	for (i=0; i<4; i++)
	{
		/* create a new node, set the node on field cache then assign a value into the coordinates field in this node 
		through field cache */
		node = nodeset.createNode(i+1, nodeTemplate1);
		fieldCache.setNode(node);
		coordinatesField.assignReal(fieldCache, /*number_of_values*/3, node_coordinates[i]);
		node = null;
	}
	fieldCache = null;
	nodeTemplate1 = null;
	
	/* Now we get a handle to the 3d mesh in the region and create an element template from the mesh*/
	var mesh = fieldModule.findMeshByDimension(3);
	var elementTemplate = mesh.createElementTemplate();
	/* set the template shape type to tetrahedron so the element to be created will be a tetrahedron */
	elementTemplate.setShapeType(elementTemplate.SHAPE_TYPE.TETRAHEDRON);
	/* set the number of nodes to 4 */
	elementTemplate.setNumberOfNodes(4);
	/* create the basis of the element to be defined */
	var tetrahedronBasis = fieldModule.createElementBasis(3, fieldModule.BASIS_FUNCTION_TYPE.LINEAR_SIMPLEX);
	/* define the coordiantes field fo the template */
	var tetrahedron_local_node_indexes = [ 1, 2, 3, 4];
	elementTemplate.defineFieldSimpleNodal(coordinatesField,
		-1, tetrahedronBasis, 4, tetrahedron_local_node_indexes);
	cubicBasis = null;
	
	/* create element */
	i = 0;
	for (i=1; i<=4; i++)
	{
		node = nodeset.findNodeByIdentifier(i);
		elementTemplate.setNode(i, node);
		node = null;
	}
	mesh.defineElement(0, elementTemplate);
	elementTemplate = null;
	mesh = null;
	nodeset = null;
	coordinatesField = null;
	fieldModule.endChange();
	field_module = null;
	region = null;
}

function createTetrahedronGraphics()
{
	var zincPlugin = document.getElementById('zincplugin');
	/* get the default region */
	var rootRegion = zincPlugin.context.getDefaultRegion();
	/* find a child region named tetrahedron and define all faces in that region */
	var child = rootRegion.findChildByName("tetrahedron");
	var childFieldModule = child.getFieldModule(); 
	childFieldModule.defineAllFaces();
	/* get the graphics module then using it to get the rendition of the tetrahedron region*/
	var graphicsModule = zincPlugin.context.getDefaultGraphicsModule();
	var rendition = graphicsModule.getRendition(child);
	drawGraphicForRendition(rendition);
	zincPlugin.sceneViewer.viewAll();	
}

/* This function is called when the primary scene is constructed and read */
function sceneViewerReadyFunction()
{
	/* an entry point to the zinc plugin */
	var zincPlugin = document.getElementById('zincplugin');
	/* define default material */
	zincPlugin.context.getDefaultGraphicsModule().defineStandardMaterials();
	createCube();
	createCubeGraphics();
	createTetrahedron();
	createTetrahedronGraphics();
}
		